#include<iostream>
#include<vector>

using namespace std;

const int MAXN = 5e5 + 5;

struct SegmentTree
{
    int val;

    int l, r;
    SegmentTree *L, *R;

    SegmentTree(){}
    SegmentTree(int l, int r)
    {
        this->val = 0;

        this->l = l;
        this->r = r;
        this->L = nullptr;
        this->R = nullptr;
    }
    SegmentTree(SegmentTree *other)
    {
        this->val = other->val;

        this->l = other->l;
        this->r = other->r;
        this->L = nullptr;
        this->R = nullptr;
    }

    void build()
    {
        if(l==r) return;

        L = new SegmentTree(l, (l+r)/2);
        R = new SegmentTree((l+r)/2+1, r);

        L->build();
        R->build();
    }

    void update(int q, int change, SegmentTree *old)
    {
        if(l==r && l==q)
        {
            val += change;
            return;
        }

        if(old->L->l<=q && q<=old->L->r)
        {
            L = new SegmentTree(old->L);
            L->update(q, change, old->L);
        }
        else
        {
            L = old->L;
        }

        if(old->R->l<=q && q<=old->R->r)
        {
            R = new SegmentTree(old->R);
            R->update(q, change, old->R);
        }
        else
        {
            R = old->R;
        }

        val = L->val + R->val;
    }

    int query(int ql, int qr)
    {
        if(ql>qr) return 0;

        if(l>=ql && r<=qr) return val;
        if(r<ql || l>qr) return 0;

        return L->query(ql, qr) + R->query(ql, qr);

    }
};

int n;
string tmp;
vector <int> graph[MAXN];

int pref[MAXN], suff[MAXN];
SegmentTree *prefT[MAXN], *suffT[MAXN], *base;

void init()
{
    base = new SegmentTree(0, 2*n);
    base->build();

    for(int i = 1;i<=n;i++)
    {
        pref[i] = pref[i-1] + ((tmp[i-1]=='+')?+1:-1);
    }

    for(int i = n;i>=1;i--)
    {
        suff[i] = suff[i+1] + ((tmp[i-1]=='+')?+1:-1);
    }

    prefT[0] = base;
    for(int i = 1;i<=n;i++)
    {
        prefT[i] = new SegmentTree(0, 2*n);
        prefT[i]->update(pref[i]+n, 1, prefT[i-1]);
    }

    suffT[n+1] = base;
    for(int i = n;i>=1;i--)
    {
        suffT[i] = new SegmentTree(0, 2*n);
        suffT[i]->update(suff[i]+n, 1, suffT[i+1]);
    }
}

int getPref(int l, int r, int val)
{
    if(l==0) return prefT[r]->query(val, 2*n);
    return prefT[r]->query(val+n, 2*n) - prefT[l-1]->query(val+n, 2*n);
}

int getSuff(int l, int r, int val)
{
    return suffT[l]->query(val+n, 2*n) - suffT[r+1]->query(val+n, 2*n);
}

int main()
{
    ios::sync_with_stdio(false);
    cin.tie(NULL);

    int Q;

    cin >> n >> Q;
    cin >> tmp;
    for(int i = 0;i<n-1;i++)
    {
        int u, v;
        cin >> u >> v;

        graph[u].push_back(v);
        graph[v].push_back(u);
    }

    init();

    while(Q--)
    {
        int u, v;
        cin >> u >> v;

        if(u<=v)
        {
            cout << getPref(u, v, pref[u-1]+1) << '\n';
        }
        else
        {
            cout << getSuff(v, u, suff[u+1]+1) << '\n';
        }
    }
}
/*
6 4
+-++--
1 2
2 3
3 4
4 5
5 6
*/
