#include<iostream>
#include<vector>

using namespace std;

const int MAXN = 5e5 + 5;

struct SegmentTree
{
    int val;

    int l, r;
    SegmentTree *L, *R;

    SegmentTree(){}
    SegmentTree(int l, int r)
    {
        this->val = 0;

        this->l = l;
        this->r = r;
        this->L = nullptr;
        this->R = nullptr;
    }
    SegmentTree(SegmentTree *other)
    {
        this->val = other->val;

        this->l = other->l;
        this->r = other->r;
        this->L = nullptr;
        this->R = nullptr;
    }

    void build()
    {
        if(l==r) return;

        L = new SegmentTree(l, (l+r)/2);
        R = new SegmentTree((l+r)/2+1, r);

        L->build();
        R->build();
    }

    void reset() //can be implemented better
    {
        val = 0;
        if(L!=nullptr) L->reset();
        if(R!=nullptr) R->reset();
    }

    void update(int q, int change)
    {
        if(l==r && l==q)
        {
            //cout << "inside " << change << '\n';

            val += change;
            return;
        }
        if(r<q || l>q) return;

        L->update(q, change);
        R->update(q, change);

        val = L->val + R->val;
    }

    int query(int ql, int qr)
    {
        if(ql>qr) return 0;

        if(l>=ql && r<=qr) return val;
        if(r<ql || l>qr) return 0;

        return L->query(ql, qr) + R->query(ql, qr);

    }
};

int n;
string tmp;
vector <int> graph[MAXN];

struct Operation
{
    int ind;
    int sign, val;

    Operation(){}
    Operation(int ind, int sign, int val)
    {
        this->ind = ind;
        this->sign = sign;
        this->val = val;
    }

    void execute(int *answer, SegmentTree *T)
    {
        // << "ind: " << sign << "*" << T->query(val+n, 2*n) << " " << val << '\n';
        answer[ind] += sign*(T->query(val+n, 2*n));
    }
};

SegmentTree *T;
int pref[MAXN], suff[MAXN];
vector <Operation> prefOp[MAXN], suffOp[MAXN];

int answer[MAXN];

void init()
{
    T = new SegmentTree(0, 2*n);
    T->build();

    for(int i = 1;i<=n;i++)
    {
        pref[i] = pref[i-1] + ((tmp[i-1]=='+')?+1:-1);
    }

    for(int i = n;i>=1;i--)
    {
        suff[i] = suff[i+1] + ((tmp[i-1]=='+')?+1:-1);
    }
}


int main()
{
    ios::sync_with_stdio(false);
    cin.tie(NULL);

    int Q;

    cin >> n >> Q;
    cin >> tmp;
    for(int i = 0;i<n-1;i++)
    {
        int u, v;
        cin >> u >> v;

        graph[u].push_back(v);
        graph[v].push_back(u);
    }

    init();

    for(int i = 0;i<Q;i++)
    {
        int u, v;
        cin >> u >> v;

        if(u<=v)
        {
            prefOp[v].push_back(Operation(i, +1, pref[u-1]+1));
            prefOp[u-1].push_back(Operation(i, -1, pref[u-1]+1));

            //cout << getPref(u, v, pref[u-1]+1) << '\n';
        }
        else
        {
            suffOp[v].push_back(Operation(i, +1, suff[u+1]+1));
            suffOp[u+1].push_back(Operation(i, -1, suff[u+1]+1));

            //cout << getSuff(v, u, suff[u+1]+1) << '\n';
        }
    }

    T = new SegmentTree(0, 2*n);
    T->build();

    for(int i = 1;i<=n;i++)
    {
        T->update(pref[i]+n, +1);
        for(Operation op: prefOp[i]) op.execute(answer, T);
    }

    T->reset();
    for(int i = n;i>=1;i--)
    {
        T->update(suff[i]+n, +1);
        for(Operation op: suffOp[i]) op.execute(answer, T);
    }

    for(int i = 0;i<Q;i++) cout << answer[i] << '\n';
}
/*
6 4
+-++--
1 2
2 3
3 4
4 5
5 6
*/
