#include <iostream>
using std::cin; using std::cout; using std::endl;

int sizeX, sizeY;
int matrix[500][500];

constexpr int dist(const int a, const int b) {
	return (a > b ? a-b : b-a);
}

struct StateField {
	int dp[10][500][500];
	char dir[10][500][500];
	bool used[500][500];
};

StateField* run() {
	StateField *sf = new StateField;

	for(int d = 0;d < 10;d ++) {
		// base case (1x1)
		sf->dp[d][0][0] = dist(d, matrix[0][0]);
		sf->dir[d][0][0] = 'x';
		//cout << d << ' ' << sf.dp[d][0][0] << endl;

		// first column
		for(int y = 1;y < sizeY;y ++) {
			sf->dp[d][0][y] = sf->dp[d][0][y-1] + dist(d, matrix[0][y]);
			sf->dir[d][0][y] = 'u';
		}

		// the rest of the matrix
		for(int x = 1;x < sizeX;x ++) {
			for(int y = 0;y < sizeY;y ++) {
				int up = (1 << 30), lf = (1 << 30);

				if(y > 0) up = sf->dp[d][x][y-1];
				lf = sf->dp[d][x-1][y];

				const int D = dist(d, matrix[x][y]);

				if(lf < up) {
					sf->dir[d][x][y] = 'l';
					sf->dp[d][x][y] = lf + D;
				} else {
					sf->dir[d][x][y] = 'u';
					sf->dp[d][x][y] = up + D;
				}
			}
		}
	}

	return sf;
}

void clearBT(StateField *sf) {
	for(int i = 0;i < 500;i ++) std::fill(sf->used[i], sf->used[i] + 500, false);
}

void backtrack(StateField *sf, const int digit, int x, int y) {
	while(true) {
		sf->used[x][y] = true;
		if(sf->dir[digit][x][y] == 'x') break;
		else if(sf->dir[digit][x][y] == 'u') y --;
		else x --;
	}
}

int calcD(StateField *sf, const int digit) {
	int res = 0;
	for(int x = 0;x < sizeX;x ++) {
		for(int y = 0;y < sizeY;y ++) {
			res += sf->used[x][y] * dist(digit, matrix[x][y]);
		}
	}
	return res;
}

int main() {
	std::cin.tie(nullptr);
	std::ios_base::sync_with_stdio(false);

	cin >> sizeY >> sizeX;
	for(int y = 0;y < sizeY;y ++) {
		for(int x = 0;x < sizeX;x ++) {
			char c; cin >> c;
			matrix[x][y] = c - '0';
		}
	}

	StateField *ul = run();

	/*
	for(int d = 0;d < 10;d ++) {
		cout << "best for D=" << d << endl;
		for(int y = 0;y < sizeY;y ++) {
			for(int x = 0;x < sizeX;x ++) {
				cout << ul->dp[d][x][y] << '\t';
			}
			cout << '\n';
		}
	}
	*/

	int ans = (1 << 30);
	for(int d = 0;d < 10;d ++) {
		for(int x = 0;x < sizeX;x ++) {
			for(int y = 0;y < sizeY;y ++) {
				clearBT(ul);
				backtrack(ul, d, x, sizeY - 1);
				backtrack(ul, d, sizeX - 1, y);
				const int total = calcD(ul, d);
				ans = std::min(total, ans);
			}
		}
	}
	cout << ans << endl;

	return 0;
}
