#include <iostream>
#include <queue>
#include <utility>
#include <algorithm>

using namespace std;

const int k_N = 500 + 3;
const int k_Inf = 1e9 + 7;

int n, m;
string s[k_N];
int dist_down[k_N][k_N][10], dist_up[k_N][k_N][10];
int dist_both[k_N][k_N][2][2][10];

struct Node{
    int x, y, digit;
    int dist;

    Node(){}
    Node(int x, int y, int digit, int dist){
        this->x = x;
        this->y = y;
        this->digit = digit;
        this->dist = dist;
    }
    friend bool operator<(const Node &lvalue, const Node &rvalue){
        return lvalue.dist > rvalue.dist;
    }
};

struct NodeBoth{
    int x, y, digit;
    bool up, down;
    int dist;

    NodeBoth(){}
    NodeBoth(int x, int y, int digit, bool up, bool down, int dist){
        this->x = x;
        this->y = y;
        this->digit = digit;
        this->dist = dist;
        this->up = up;
        this->down = down;
    }
    friend bool operator<(const NodeBoth &lvalue, const NodeBoth &rvalue){
        return lvalue.dist > rvalue.dist;
    }
};

bool valid(int x, int y){
    return !(x < 0 || y < 0 || x >= n || y >= m);
}

void dijkstra_down(){
    priority_queue<Node> pq;
    for(int i = 0; i < n; ++i)
        for(int j = 0; j < m; ++j)
            for(int digit = 0; digit < 10; ++digit)
                dist_down[i][j][digit] = k_Inf;

    for(int j = 0; j < m; ++j){
        for(int digit = 0; digit < 10; ++digit){
            pq.push(Node(n - 1, j, digit, abs(digit - s[n - 1][j])));
            dist_down[n - 1][j][digit] = abs(digit - s[n - 1][j]);
        }
    }

    while(!pq.empty()){
        Node node = pq.top();
        pq.pop();

        if(node.dist != dist_down[node.x][node.y][node.digit])
            continue;

        int x = node.x;
        int y = node.y;
        int dist = node.dist;
        int digit = node.digit;

        vector<pair<int, int> > adj;
        adj.push_back(make_pair(x + 1, y));
        adj.push_back(make_pair(x - 1, y));
        adj.push_back(make_pair(x, y + 1));
        adj.push_back(make_pair(x, y - 1));

        for(int i = 0; i < 4; ++i){
            pair<int, int> p = adj[i];
            int x_to = p.first;
            int y_to = p.second;

            if(!valid(x_to, y_to)) continue;

            int new_dist = dist + abs(digit - s[x_to][y_to]);
            if(new_dist < dist_down[x_to][y_to][digit]){
                dist_down[x_to][y_to][digit] = new_dist;
                pq.push(Node(x_to, y_to, digit, new_dist));
            }
        }
    }
}

void dijkstra_up(){
    priority_queue<Node> pq;
    for(int i = 0; i < n; ++i)
        for(int j = 0; j < m; ++j)
            for(int digit = 0; digit < 10; ++digit)
                dist_up[i][j][digit] = k_Inf;

    for(int j = 0; j < m; ++j){
        for(int digit = 0; digit < 10; ++digit){
            pq.push(Node(0, j, digit, abs(digit - s[0][j])));
            dist_up[0][j][digit] = abs(digit - s[0][j]);
        }
    }

    while(!pq.empty()){
        Node node = pq.top();
        pq.pop();

        if(node.dist != dist_up[node.x][node.y][node.digit])
            continue;

        int x = node.x;
        int y = node.y;
        int dist = node.dist;
        int digit = node.digit;

        vector<pair<int, int> > adj;
        adj.push_back(make_pair(x + 1, y));
        adj.push_back(make_pair(x - 1, y));
        adj.push_back(make_pair(x, y + 1));
        adj.push_back(make_pair(x, y - 1));

        for(int i = 0; i < 4; ++i){
            pair<int, int> p = adj[i];
            int x_to = p.first;
            int y_to = p.second;

            if(!valid(x_to, y_to)) continue;

            int new_dist = dist + abs(digit - s[x_to][y_to]);
            if(new_dist < dist_up[x_to][y_to][digit]){
                dist_up[x_to][y_to][digit] = new_dist;
                pq.push(Node(x_to, y_to, digit, new_dist));
            }
        }
    }
}

void dijkstra_both(){
    priority_queue<NodeBoth> pq;
    for(int i = 0; i < n; ++i)
        for(int j = 0; j < m; ++j)
            for(int digit = 0; digit < 10; ++digit)
                for(int up = 0; up < 2; ++up)
                    for(int down = 0; down < 2; ++down)
                        dist_both[i][j][up][down][digit] = k_Inf;

    ///important 7
    for(int i = 0; i < n; ++i){
        for(int digit = 0; digit < 10; ++digit){
            pq.push(NodeBoth(i, m - 1, digit, false, false, abs(digit - s[i][m - 1])));
            dist_both[i][m - 1][0][0][digit] = abs(digit - s[i][m - 1]);
        }
    }

    while(!pq.empty()){
        NodeBoth node = pq.top();
        pq.pop();

        int x = node.x;
        int y = node.y;
        int dist = node.dist;
        int digit = node.digit;
        bool up = node.up;
        bool down = node.down;

        //cout << "pq -" << x << " " << y << " " << dist << " " << digit << " " << up << " " << down << "\n";

        if(dist != dist_both[x][y][up][down][digit])
            continue;



        vector<pair<int, int> > adj;
        adj.push_back(make_pair(x + 1, y));
        adj.push_back(make_pair(x - 1, y));
        adj.push_back(make_pair(x, y + 1));
        adj.push_back(make_pair(x, y - 1));

        for(int i = 0; i < 4; ++i){
            pair<int, int> p = adj[i];
            int x_to = p.first;
            int y_to = p.second;

            if(!valid(x_to, y_to)) continue;

            int new_dist = dist + abs(digit - s[x_to][y_to]);
            if(new_dist < dist_both[x_to][y_to][up][down][digit]){
                dist_both[x_to][y_to][up][down][digit] = new_dist;
                pq.push(NodeBoth(x_to, y_to, digit, up, down, new_dist));
            }
        }

        if(!up){
            int new_dist = dist - abs(digit - s[x][y]) + dist_up[x][y][digit];
            if(new_dist < dist_both[x][y][true][down][digit]){
                dist_both[x][y][true][down][digit] = new_dist;
                pq.push(NodeBoth(x, y, digit, true, down, new_dist));
            }
        }
        if(!down){
            int new_dist = dist - abs(digit - s[x][y]) + dist_down[x][y][digit];
            if(new_dist < dist_both[x][y][up][true][digit]){
                dist_both[x][y][up][true][digit] = new_dist;
                pq.push(NodeBoth(x, y, digit, up, true, new_dist));
            }
        }
    }
}

int main(){
    ios::sync_with_stdio(false);
    cin.tie(NULL);

    cin >> n >> m;

    for(int i = 0; i < n; ++i)
        cin >> s[i];

    for(int i = 0; i < n; ++i)
        for(int j = 0; j < m; ++j)
            s[i][j] -= '0';

    dijkstra_up();
    dijkstra_down();
    dijkstra_both();

    int ans = k_Inf;
    for(int i = 0; i < n; ++i){
        for(int digit = 0; digit < 10; ++digit){
            ans = min(ans, dist_both[i][0][true][true][digit]);
            //cout << dist_both[i][0][true][true][digit] << " - " << i << " " << digit << "\n";
        }
    }

    /*cout << "down:\n";
    for(int i = 0; i < n; ++i){
        for(int j = 0; j < m; ++j){
            cout << dist_down[i][j][7] << " ";
        }
        cout << "\n";
    }

    cout << "up\n";
    for(int i = 0; i < n; ++i){
        for(int j = 0; j < m; ++j){
            cout << dist_up[i][j][7] << " ";
        }
        cout << "\n";
    }

    cout << "both\n";
    for(int i = 0; i < n; ++i){
        for(int j = 0; j < m; ++j){
            cout << dist_both[i][j][true][true][7] << " ";
        }
        cout << "\n";
    }*/

    cout << ans << "\n";
}
