#include <iostream>
#include <vector>

using namespace std;

const int k_N = 5e5 + 3;
const int k_MAX = 1e7 + 3;

struct PersistentSegmentTree{
    int sum[k_MAX], lch[k_MAX], rch[k_MAX];
    int sz;
    int n;

    PersistentSegmentTree(){
        sz = 0;
    }

    void init(int idx, int l, int r){
        if(l == r){
            sum[idx] = 0;
            return;
        }

        lch[idx] = sz++;
        rch[idx] = sz++;

        int mid = (l + r) >> 1;
        init(lch[idx], l, mid);
        init(rch[idx], mid + 1, r);

        sum[idx] = 0;
    }

    void resize(int n){
        this->n = n;
        sz++;
        init(sz - 1, 1, n);
    }

    int update(int idx, int l, int r, int s){
        if(s < l || r < s)
            return idx;
        if(l == r){
            sum[sz] = 1;
            return sz++;
        }

        int mid = (l + r) >> 1;
        int new_lch = update(lch[idx], l, mid, s);
        int new_rch = update(rch[idx], mid + 1, r, s);

        lch[sz] = new_lch;
        rch[sz] = new_rch;
        sum[sz] = sum[new_lch] + sum[new_rch];

        return sz++;
    }

    int query(int idx, int l, int r, int sl, int sr){
        if(sl > r || sr < l)
            return 0;
        if(sl <= l && r <= sr)
            return sum[idx];

        int mid = (l + r) >> 1;
        return query(lch[idx], l, mid, sl, sr) + query(rch[idx], mid + 1, r, sl, sr);
    }
};

int n, q;
int val[k_N];
vector<int> adj[k_N];

int prefix[k_N], suffix[k_N];
PersistentSegmentTree suffix_st, prefix_st;

vector<int> suffix_pos[2][k_N], prefix_pos[2][k_N];
int suffix_st_idx[2][k_N], prefix_st_idx[2][k_N];

int main(){
    ios::sync_with_stdio(false);
    cin.tie(NULL);

    cin >> n >> q;
    {
        string s;
        cin >> s;

        for(int i = 0; i < n; ++i){
            if(s[i] == '+')
                val[i + 1] = 1;
            else
                val[i + 1] = -1;
        }
    }

    for(int i = 0; i < n - 1; ++i){
        int u, v;
        cin >> u >> v;
        adj[u].push_back(v);
        adj[v].push_back(u);
    }

    for(int i = 1; i <= n; ++i)
        prefix[i] = prefix[i - 1] + val[i];
    for(int i = n; i >= 1; --i)
        suffix[i] = suffix[i + 1] + val[i];

    for(int i = 1; i <= n; ++i){
        bool f = suffix[i] >= 0;
        int s = abs(suffix[i]);
        suffix_pos[f][s].push_back(i);

        f = prefix[i] >= 0;
        s = abs(prefix[i]);
        prefix_pos[f][s].push_back(i);
    }

    prefix_st.resize(n);
    suffix_st.resize(n);

    int curr_idx = 0;
    for(int i = k_N - 1; i > -k_N; --i){
        bool f = (i >= 0);
        int s = abs(i);
        prefix_st_idx[f][s] = curr_idx;

        for(int j = 0; j < prefix_pos[f][s].size(); ++j){
            int x = prefix_pos[f][s][j];
            //cout << i << " - " << x << "\n";
            curr_idx = prefix_st.update(curr_idx, 1, n, x);
        }
    }

    curr_idx = 0;
    for(int i = k_N - 1; i > -k_N; --i){
        bool f = (i >= 0);
        int s = abs(i);
        suffix_st_idx[f][s] = curr_idx;

        for(int j = 0; j < suffix_pos[f][s].size(); ++j){
            int x = suffix_pos[f][s][j];
            curr_idx = suffix_st.update(curr_idx, 1, n, x);
        }
    }

    //for(int i = 1; i <= n; ++i)
    //    cout << prefix[i] << " ";
    //cout << "\n";

    for(int i = 0; i < q; ++i){
        int a, b;
        cin >> a >> b;

        if(a <= b){
            int start = prefix[a] - val[a];
            bool f = (start >= 0);
            int s = abs(start);
            cout << prefix_st.query(prefix_st_idx[f][s], 1, n, a, b) << "\n";
        }
        else{
            int start = suffix[a] - val[a];
            bool f = (start >= 0);
            int s = abs(start);
            cout << suffix_st.query(suffix_st_idx[f][s], 1, n, b, a) << "\n";
        }
    }
}
