#include <iostream>
#include <vector>

using namespace std;

const int k_N = 5e5 + 3;
const int k_MAX = 1e7 + 1e6;
const int k_LOG = 19;

struct PersistentSegmentTree{
    int sum[k_MAX], lch[k_MAX], rch[k_MAX];
    int sz;
    int n;

    PersistentSegmentTree(){
        sz = 0;
    }

    void init(int idx, int l, int r){
        if(l == r){
            sum[idx] = 0;
            return;
        }

        lch[idx] = sz++;
        rch[idx] = sz++;

        int mid = (l + r) >> 1;
        init(lch[idx], l, mid);
        init(rch[idx], mid + 1, r);

        sum[idx] = 0;
    }

    void resize(int n){
        this->n = n;
        sz++;
        init(sz - 1, 1, n);
    }

    int update(int idx, int l, int r, int s){
        if(s < l || r < s)
            return idx;
        if(l == r){
            sum[sz] = 1;
            return sz++;
        }

        int mid = (l + r) >> 1;
        int new_lch = update(lch[idx], l, mid, s);
        int new_rch = update(rch[idx], mid + 1, r, s);

        lch[sz] = new_lch;
        rch[sz] = new_rch;
        sum[sz] = sum[new_lch] + sum[new_rch];

        return sz++;
    }

    int query(int idx, int l, int r, int sl, int sr){
        if(sl > r || sr < l)
            return 0;
        if(sl <= l && r <= sr)
            return sum[idx];

        int mid = (l + r) >> 1;
        return query(lch[idx], l, mid, sl, sr) + query(rch[idx], mid + 1, r, sl, sr);
    }
};

int n, q;
int val[k_N];
vector<int> adj[k_N];

int prefix[k_N], suffix[k_N];
PersistentSegmentTree suffix_st, prefix_st;

vector<int> suffix_pos[2][k_N], prefix_pos[2][k_N];
int suffix_st_idx[2][k_N], prefix_st_idx[2][k_N];

int head[k_N], sz[k_N], d[k_N];
int par[k_LOG][k_N];
int st[k_N], en[k_N], timer;

int arr[k_N];

void dfs(int u, int pr){
    sz[u] = 1;
    d[u] = d[pr] + 1;
    for(int i = 0; i < adj[u].size(); ++i){
        int to = adj[u][i];
        if(to == pr) continue;
        dfs(to, u);
        sz[to] += sz[u];
    }
}

void decompose(int u, int pr, int h){
    head[u] = h;
    st[u] = ++timer;
    arr[st[u]] = u;

    par[0][u] = pr;
    for(int i = 1; i < k_LOG; i++)
        par[i][u] = par[i - 1][par[i - 1][u]];

    int mx_tr = 0, node;
    for(int i = 0; i < adj[u].size(); ++i){
        int to = adj[u][i];
        if(to == pr) continue;

        if(sz[to] > mx_tr){
            node = to;
            mx_tr = sz[to];
        }
    }

    if(!mx_tr){
        int from = st[head[u]];
        int to = st[u];

        prefix[from] = val[arr[from]];
        for(int i = from + 1; i <= to; ++i)
            prefix[i] = prefix[i - 1] + val[arr[i]];

        suffix[to] = val[arr[to]];
        for(int i = to - 1; i >= from; --i)
            suffix[i] = suffix[i + 1] + val[arr[i]];

        return;
    }

    decompose(node, u, h);

    for(int i = 0; i < adj[u].size(); ++i){
        int to = adj[u][i];
        if(to == pr || to == node) continue;

        decompose(to, u, to);
    }

    en[u] = timer;
}

int get_lca(int a, int b){
    if(d[a] != d[b]){
        if(d[a] < d[b])
            swap(a, b);

        int diff = d[a] - d[b];
        for(int i = k_LOG - 1; i >= 0; --i)
            if(diff & (1 << i))
                a = par[i][a];
    }

    if(a == b) return a;

    for(int i = k_LOG - 1; i >= 0; --i){
        if(par[i][a] != par[i][b]){
            a = par[i][a];
            b = par[i][b];
        }
    }

    return par[0][a];
}

vector<pair<int, int> > get_path_up(int u, int anc){
    vector<pair<int, int> > ans;
    while(st[u] > st[anc]){
        ans.push_back(make_pair(max(st[anc] + 1, st[head[u]]), st[u]));
        u = par[0][head[u]];
    }
    return ans;
}

int solve(int a, int b){
    int lca = get_lca(a, b);

    vector<pair<int, int> > v1 = get_path_up(a, lca);
    vector<pair<int, int> > v2 = get_path_up(b, lca);

    int ans = 0;
    int curr = 0;
    for(int i = 0; i < v1.size(); ++i){
        pair<int, int> p = v1[i];
        int t = -curr + suffix[p.second] - val[arr[p.second]];
        bool f = t >= 0;
        int s = abs(t);

        int idx = suffix_st_idx[f][s];
        ans += suffix_st.query(idx, 1, n, p.first, p.second);

        curr += prefix[p.second] - prefix[p.first] + val[arr[p.first]];
    }

    ans += val[lca] > -curr;
    curr += val[lca];

    for(int i = (int)v2.size() - 1; i >= 0; --i){
        pair<int, int> p = v2[i];
        int t = -curr + prefix[p.first] - val[arr[p.first]];
        bool f = t >= 0;
        int s = abs(t);

        //cout << t << " - t" << endl;
        int idx = prefix_st_idx[f][s];
        ans += prefix_st.query(idx, 1, n, p.first, p.second);

        curr += prefix[p.second] - prefix[p.first] + val[arr[p.first]];
    }

    return ans;
}

int main(){
    ios::sync_with_stdio(false);
    cin.tie(NULL);

    cin >> n >> q;
    {
        string s;
        cin >> s;

        for(int i = 0; i < n; ++i){
            if(s[i] == '+')
                val[i + 1] = 1;
            else
                val[i + 1] = -1;
        }
    }

    for(int i = 0; i < n - 1; ++i){
        int u, v;
        cin >> u >> v;
        adj[u].push_back(v);
        adj[v].push_back(u);
    }

    dfs(1, 1);
    decompose(1, 1, 1);

    for(int i = 1; i <= n; ++i){
        bool f = suffix[i] >= 0;
        int s = abs(suffix[i]);
        suffix_pos[f][s].push_back(i);

        f = prefix[i] >= 0;
        s = abs(prefix[i]);
        prefix_pos[f][s].push_back(i);
    }

    prefix_st.resize(n);
    suffix_st.resize(n);

    int max_prefix = 0, min_prefix = 0;
    for(int i = 1; i <= n; ++i){
        max_prefix = max(prefix[i], max_prefix);
        min_prefix = min(prefix[i], min_prefix);
    }

    int max_suffix = 0, min_suffix = 0;
    for(int i = 1; i <= n; ++i){
        max_suffix = max(suffix[i], max_suffix);
        min_suffix = min(suffix[i], min_suffix);
    }

    int curr_idx = 0;
    for(int i = max_prefix; i >= min_prefix; --i){
        bool f = (i >= 0);
        int s = abs(i);
        prefix_st_idx[f][s] = curr_idx;

        for(int j = 0; j < prefix_pos[f][s].size(); ++j){
            int x = prefix_pos[f][s][j];
            //cout << i << " - " << x << "\n";
            curr_idx = prefix_st.update(curr_idx, 1, n, x);
        }
    }

    curr_idx = 0;
    for(int i = max_suffix; i >= min_suffix; --i){
        bool f = (i >= 0);
        int s = abs(i);
        suffix_st_idx[f][s] = curr_idx;

        for(int j = 0; j < suffix_pos[f][s].size(); ++j){
            int x = suffix_pos[f][s][j];
            curr_idx = suffix_st.update(curr_idx, 1, n, x);
        }
    }

    //for(int i = 1; i <= n; ++i)
    //    cout << prefix[i] << " ";
    //cout << "\n";

    for(int i = 0; i < q; ++i){
        int a, b;
        cin >> a >> b;

        cout << solve(a, b) << "\n";
    }
}
/*
7 3
-+-++--
1 2
1 3
2 4
2 7
3 6
3 5
4 6
6 4
5 7
*/
