#include <bits/stdc++.h>

#pragma GCC optimize "-O3"

#define endl '\n'
#define trace(x) cerr << #x << " = " << x <<endl

using namespace std;

const int INF = 1 << 30;

void set_io() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    cout.tie(nullptr);
}

int n, m;
vector<string> grid;

void read() {
    cin >> n >> m;

    grid.resize(n);
    for (string &s : grid) {
        cin >> s;
    }
}

int dx[4] = {0, -1, 0, 1};
int dy[4] = {-1, 0, 1, 0};

inline bool is_out(int x, int y) {
    return (x < 0 || x >= n || y < 0 || y >= m);
}

vector<vector<int>> costs;
vector<vector<bool>> seen;
vector<vector<pair<int, int>>> came_from;

void bfs(int x, int y, char target) {
    for (int i = 0; i < n; i++) {
        fill(costs[i].begin(), costs[i].end(), INF);
        fill(seen[i].begin(), seen[i].end(), false);
    }

    priority_queue<pair<int, pair<int, int>>> heap;
    int initial_cost = abs(grid[x][y] - target);

    costs[x][y] = initial_cost;
    came_from[x][y] = {-1, -1};
    heap.emplace(-initial_cost, make_pair(x, y));

    while (!heap.empty()) {
        auto now = heap.top();
        heap.pop();

        if (seen[now.second.first][now.second.second]) {
            continue;
        }

        seen[now.second.first][now.second.second] = true;

        for (int dir = 0; dir < 4; dir++) {
            int next_x = now.second.first + dx[dir];
            int next_y = now.second.second + dy[dir];

            if (!is_out(next_x, next_y) && !seen[next_x][next_y]) {
                int cost = abs(grid[next_x][next_y] - target);

                if (-now.first + cost < costs[next_x][next_y]) {
                    costs[next_x][next_y] = -now.first + cost;
                    came_from[next_x][next_y] = now.second;
                    heap.emplace(-costs[next_x][next_y], make_pair(next_x, next_y));
                }
            }
        }
    }
}

vector<vector<int>> used;

int mark_path(pair<int, int> pos, char target, int id) {
    int cost = 0;

    while (pos.first != -1 && !used[pos.first][pos.second]) {
        used[pos.first][pos.second] = id;
        cost += abs(grid[pos.first][pos.second] - target);
        pos = came_from[pos.first][pos.second];
    }

    return cost;
}

int redo_path(pair<int, int> pos, char target, int id) {
    int cost = 0;

    while (pos.first != -1 && used[pos.first][pos.second] == id) {
        used[pos.first][pos.second] = 0;
        cost += abs(grid[pos.first][pos.second] - target);
        pos = came_from[pos.first][pos.second];
    }

    return cost;
}

int main() {
    set_io();

    read();

    costs.resize(n, vector<int>(m));
    seen.resize(n, vector<bool>(m));
    came_from.resize(n, vector<pair<int, int>>(m));
    used.resize(n, vector<int>(m, 0));

    int best_time = INF;

    for (char target_digit = '0'; target_digit <= '9'; target_digit++) {
        for (int top_column = 1; top_column < m - 1; top_column++) {
            bfs(0, top_column, target_digit);

            int time = mark_path({0, top_column}, target_digit, 1);
            for (int left_row = 0; left_row < n; left_row++) {
                time += mark_path({left_row, 0}, target_digit, 2);

                for (int right_row = 0; right_row < n; right_row++) {
                    time += mark_path({left_row, m - 1}, target_digit, 3);

                    for (int bottom_column = 1; bottom_column < m - 1; bottom_column++) {
                        time += mark_path({n - 1, bottom_column}, target_digit, 4);

                        best_time = min(best_time, time);

                        time -= redo_path({n - 1, bottom_column}, target_digit, 4);
                    }

                    time += redo_path({left_row, m - 1}, target_digit, 3);
                }

                time -= redo_path({left_row, 0}, target_digit, 2);
            }
        }
    }

    cout << best_time << endl;

    return EXIT_SUCCESS;
}
