#include <bits/stdc++.h>

#pragma GCC optimize "-O3"

#define endl '\n'
#define trace(x) cerr << #x << " = " << x <<endl

using namespace std;

void set_io() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    cout.tie(nullptr);
}

int n, q;
string types;
vector<vector<int>> adj_list;

void read() {
    cin >> n >> q;

    cin >> types;
    types = ' ' + types;

    adj_list.resize(n + 1);
    for (int i = 0; i < n - 1; i++) {
        int u, v;
        cin >> u >> v;

        adj_list[u].emplace_back(v);
        adj_list[v].emplace_back(u);
    }
}

vector<int> prefix_sum;
vector<int> parent;

void dfs_sum(int root) {
    prefix_sum[root] = (types[root] == '+' ? 1 : -1);
    if (parent[root] != -1) {
        prefix_sum[root] += prefix_sum[parent[root]];
    }

    for (int adj_node : adj_list[root]) {
        if (adj_node != parent[root]) {
            parent[adj_node] = root;
            dfs_sum(adj_node);
        }
    }
}

vector<int> tour, level, first;

void dfs_lca(int root, int depth = 0) {
    tour.emplace_back(root);
    level.emplace_back(depth);
    if (first[root] == -1) {
        first[root] = tour.size() - 1;
    }

    for (int adj_node : adj_list[root]) {
        if (adj_node != parent[root]) {
            dfs_lca(adj_node, depth + 1);

            tour.emplace_back(root);
            level.emplace_back(depth);
        }
    }
}

vector<int> tree;

int merge(int left, int right) {
    return ((level[left] <= level[right]) ? left : right);
}

void build(int node, int low, int high) {
    if (low == high) {
        tree[node] = low;
        return;
    }

    int mid = (low + high) / 2;
    build(node * 2, low, mid);
    build(node * 2 + 1, mid + 1, high);

    tree[node] = merge(tree[node * 2], tree[node * 2 + 1]);
}

int query(int node, int low, int high, int ql, int qr) {
    if (low >= ql && high <= qr) {
        return tree[node];
    }

    if (high < ql || low > qr) {
        return -1;
    }

    int mid = (low + high) / 2;
    int p1 = query(node * 2, low, mid, ql, qr);
    int p2 = query(node * 2 + 1, mid + 1, high, ql, qr);

    if (p1 == -1) {
        return p2;
    }

    if (p2 == -1) {
        return p1;
    }

    return merge(p1, p2);
}

int lca(int a, int b) {
    if (first[a] > first[b]) {
        swap(a, b);
    }

    return tour[query(1, 0, level.size() - 1, first[a], first[b])];
}

int query(int from, int to) {
    int result = prefix_sum[from];
    if (to > 1) {
        result -= prefix_sum[parent[to]];
    }

    return result;
}

int main() {
    set_io();

    read();

    parent.resize(n + 1, -1);
    prefix_sum.resize(n + 1);
    dfs_sum(1);

    first.resize(n + 1, -1);
    dfs_lca(1);

    tree.resize(2 * level.size());
    build(1, 0, level.size() - 1);

    while (q--) {
        int a, b;
        cin >> a >> b;

        int lca_node = lca(a, b);
        int from = a;

        int cnt = 0;
        while (a != lca_node && b != lca_node) {
            if (query(from, a) > 0) {
                cnt++;
            }

            if (query(from, lca_node) + query(b, lca_node) - query(lca_node, lca_node) > 0) {
                cnt++;
            }

            a = parent[a];
            b = parent[b];
        }

        while (a != lca_node) {
            if (query(from, a) > 0) {
                cnt++;
            }

            a = parent[a];
        }

        while (b != lca_node) {
            if (query(from, lca_node) + query(b, lca_node) - query(lca_node, lca_node) > 0) {
                cnt++;
            }

            b = parent[b];
        }

        cnt += (query(from, lca_node) > 0);

        cout << cnt << endl;
    }

    return EXIT_SUCCESS;
}
