#include <iostream>
#include <vector>
#include <string>
#include <algorithm>
#include <map>
#define fi first
#define se second
using namespace std;

const int MAXN = 500005;

int n, q;
vector<int> g[MAXN];
string s, ss;

bool subtask2 = true;
pair<int, int> queries[MAXN];
int answers[MAXN];

void Init()
{
    ios :: sync_with_stdio(false);
    cin.tie(NULL); cout.tie(NULL);

    cin >> n >> q;
    cin >> s;
    ss = s;
    reverse(ss.begin(), ss.end());
    ss = "0" + ss;
    s = "0" + s;
    for (int i = 1; i < n; ++ i)
    {
        int u, v;
        cin >> u >> v;
        g[u].push_back(v);
        g[v].push_back(u);
        if (u != i and v != i + 1) subtask2 = false;
    }

    for (int i = 1; i <= q; ++ i)
    {
        cin >> queries[i].fi >> queries[i].se;
    }
}

void DFS(int u, int t, int p, int res, int tmp)
{
    if (s[u] == '+') tmp++;
    else tmp--;
    if (tmp > 0) res++;

    if (u == t)
    {
        cout << res << endl;
        return;
    }

    for (int v : g[u])
    {
        if (v != p) DFS(v, t, u, res, tmp);
    }
}

struct node
{
    int lch, rch, val;
};

vector<node> tree;
int roots[MAXN], root = -1;

void update(int x, int v, int l = 1, int r = n)
{
    int idx = v;
    bool created = false;
    if (tree[idx].lch == 0 or tree[idx].rch == 0)
    {
        created = true;
        tree[idx].lch = tree.size();
        tree.emplace_back();
        tree[idx].rch = tree.size();
        tree.emplace_back();
    }

    //cout << v << " -> " << idx << ' ' << tree[idx].lch << ' ' << tree[idx].rch << endl;

    if (x < l or x > r) return;
    if (x == l and x == r)
    {
        tree[idx].val++;
        //cout << "update: " << idx << ' ' << l << ' ' << r << " becomes " << tree[idx].val << endl;
        return;
    }

    int mid = (l + r) / 2;
    if (x <= mid)
    {
        if (!created)
        {
            tree.emplace_back();
            tree.back() = tree[tree[idx].lch];
            tree[idx].lch = tree.size() - 1;
        }
        update(x, tree[idx].lch, l, mid + 0);
    }
    else
    {
        if (!created)
        {
            tree.emplace_back();
            tree.back() = tree[tree[idx].rch];
            tree[idx].rch = tree.size() - 1;
        }
        update(x, tree[idx].rch, mid + 1, r);
    }
    tree[idx].val = tree[tree[idx].lch].val + tree[tree[idx].rch].val;

    //cout << "update: " << idx << ' ' << l << ' ' << r << " becomes " << tree[idx].val << endl;

}

int query(int v, int le, int ri, int l = 1, int r = n)
{
    //cout << "query: " << v << ' ' << le << ' ' << ri << ' ' << l << ' ' << r << ' ' << tree[v].val << endl;

    if (ri < l or le > r) return 0;
    if (le <= l and r <= ri) return tree[v].val;

    if (tree[v].lch == 0 or tree[v].rch == 0) return 0;

    int mid = (l + r) / 2;
    return query(tree[v].lch, le, ri, l, mid + 0) + query(tree[v].rch, le, ri, mid + 1, r);
}

pair<int, int> p[MAXN];
map<int, int> m;

bool comp(pair<int, int> x, pair<int, int> y)
{
    return x.se < y.se;
}

void SolveST2()
{
    //cout << "\nSubtask 2" << endl;

    int maxp = -1;

    p[0].fi = 0;
    for (int i = 1; i <= n; ++ i)
    {
        if (s[i] == '+') p[i].fi = p[i - 1].fi + 1;
        else p[i].fi = p[i - 1].fi - 1;
        p[i].se = i;

        maxp = max(p[i].fi, maxp);
    }

    sort(p + 1, p + n + 1);

    for (int i = n; i >= 1; -- i)
    {
        //cout << p[i].fi << ' ' << p[i].se << endl;

        tree.emplace_back();
        if (root != -1) tree.back() = tree[root];
        root = tree.size() - 1;
        update(p[i].se, root);

        m[p[i].fi] = n - i;
        roots[n - i] = root;
    }

    sort(p + 1, p + n + 1, comp);

    for (int i = 1; i <= q; ++ i)
    {
        int a, b;
        a = queries[i].fi;
        b = queries[i].se;
        if (a > b) continue;

        int gr = p[a - 1].fi + 1;
        if (gr > maxp)
        {
            answers[i] = 0;
            continue;
        }

        gr = m[gr];

        //cout << "answering " << a << ' ' << b << endl;
        //cout << "not less than " << p[a - 1].fi + 1 << ' ' << maxp << endl;
        //cout << "root " << roots[gr] << endl;
        answers[i] = query(roots[gr], a, b);
        //cout << "res " << answers[i] << endl;
    }
}

int main()
{
    Init();

    if (subtask2)
    {
        SolveST2();
        s = ss;
        for (int i = 1; i <= q; ++ i)
        {
            queries[i].fi = n - queries[i].fi + 1;
            queries[i].se = n - queries[i].se + 1;
        }
        root = -1;
        tree.resize(0);
        m.clear();
        SolveST2();

        for (int i = 1; i <= q; ++ i)
        {
            cout << answers[i] << '\n';
        }

        return 0;
    }

    for (int i = 1; i <= q; ++ i)
    {
        int a, b;
        cin >> a >> b;
        DFS(a, b, 0, 0, 0);
    }
}
